
;;  -------------------------------------------------------------------
;;  NOM:        PANTALLA.ASM
;;  CREAT PER:  Sergio Blanco Cuaresma
;;		Francesc Galiano Baixauli
;;  DATA:       17/05/2001
;;  DESCRIPCIO: fitxer en "ASM" amb subrutines d'accs a pantalla en
;;              mode text 80x25.
;;  -------------------------------------------------------------------



PUBLIC _IniPantalla, _EscriureCar
PUBLIC _EscriureString, _EsborrarPantalla, _QuinCarAtr



_DATA   SEGMENT WORD PUBLIC 'DATA'
_DATA   ENDS


_BSS    SEGMENT WORD PUBLIC 'BSS'

        SegmentPant DW ?        ; Variable que contindr el segment de
                                ; memria on est mapejada la pantalla
_BSS    ENDS


DGROUP  GROUP   _DATA, _BSS


ASSUME cs:PANTALLA_CODI, ds:DGROUP


PANTALLA_CODI SEGMENT BYTE PUBLIC 'CODE'

;;  *******************************************************************
;;    ADR IniPantalla (void);
;;
;;          Subrutina d'inicialitzaci:
;;              esbrina a quina adrea de memria es troba mapejada la
;;              pantalla, retornant aquest valor com a resultat i tamb
;;              guardant una cpia del valor del segment a una variable
;;              que s'utilitza a la resta de subrutines del mdul.
;;              En cas que el sistema no estigui en mode text 80x25, es
;;              retorna el valor especial 0000:0000.
;;  *******************************************************************

_IniPantalla    PROC FAR
        push    es

        mov     ax, 0
        mov     es, ax
        mov     al, es:[0449h]          ; AL <- Mode Video Actiu
        cmp     al, 7
        je      ModeMonocrom
        cmp     al, 2
        je      ModeColor
        cmp     al, 3
        je      ModeColor

ModeErroni:
        mov     dx, 0
        jmp     Sortida

ModeColor:
        mov     dx, 0B800h
        jmp     Sortida

ModeMonocrom:
        mov     dx, 0B000h

Sortida:
        mov     SegmentPant, dx
        mov     ax, 0

        pop     es
        ret
_IniPantalla    ENDP



;;  *******************************************************************
;;    void EscriureCar (BYTE fila, BYTE columna, char caracter, BYTE atribut);
;;
;;          Subrutina d'escriptura d'un carcter per pantalla:
;;              escriu a pantalla el carcter que se li passa, amb l'atri-
;;              but indicat, a les coordenades especificades. La informaci
;;              d'on es troba mapejada la pantalla, l'obt de SegmentPant.
;;  *******************************************************************

_EscriureCar    PROC FAR
        push    bp
        mov     bp, sp
        push    ax
        push    bx
        push    es

            ; Clcul del desplaament respecte a l'inici de pantalla
        mov     al, [bp+6]      ; AL <- fila
        mov     bl, 80
        mul     bl              ; AX <- fila*80
        mov     bl, [bp+8]      ; BL <- columna
        mov     bh, 0           ; BX <- columna
        add     bx, ax          ; BX <- fila*80+columna
        shl     bx, 1           ; BX <- (fila*80+columna)*2
        mov     es, SegmentPant

            ; Escriptura del carcter
        mov     al, [bp+10]     ; AL <- caracter
        mov     ah, [bp+12]     ; AH <- atribut
        mov     es:[bx], ax

        pop     es
        pop     bx
        pop     ax
        pop     bp
        ret
_EscriureCar    ENDP



;;  *******************************************************************
;;    void EscriureString (BYTE fila,BYTE columna,char * string,BYTE atribut);
;;
;;          Subrutina d'escriptura d'un "string" per pantalla:
;;              escriu a pantalla el "string" que se li passa, amb l'atri-
;;              but indicat, a les coordenades especificades. La informaci
;;              d'on es troba mapejada la pantalla, l'obt de SegmentPant.
;;              La seva implementaci pot utilitzar les instruccions ASM
;;              de tractament de "strings" LODSB i STOSW.
;;  *******************************************************************

_EscriureString PROC FAR
        push    bp
        mov     bp, sp
        push    ax		;Guardem registres que modificarem
        push    bx
        push    es
        push	di
        push	si
        push	ds

            ; Clcul del desplaament respecte a l'inici de pantalla
        mov     al, [bp+6]      ; AL <- fila
        mov     bl, 80
        mul     bl              ; AX <- fila*80
        mov     bl, [bp+8]      ; BL <- columna
        mov     bh, 0           ; BX <- columna
        add     bx, ax          ; BX <- fila*80+columna
        shl     bx, 1           ; BX <- (fila*80+columna)*2
        mov	di, bx		; Pasem a DS per utilitzar STOSW
        mov     es, SegmentPant
		; En es:[di] tenim la posicio de pantalla (stow)
	mov	si, [bp+10]		;OFFSET de l'String
        mov	bx, [bp+12]		;SEGMENT de l'String
        mov	ds, bx
        	; En ds:[si] tenim l'inici de l'string (lodsb)
        mov	ah, [bp+14]		;Atribut
        cld			;Direccio positiva
string: lodsb				;Carguem a AL el caracter a imprimir
        cmp	al, 0			;Comprobem que no es final d'string
        je	finstring
        stosw				;Posem l'atribut i el caracter (AX) a pantalla
        jmp string
finstring:
	pop	ds			;Restaurem registres modificats
	pop 	si
	pop	di
	pop	es
	pop	bx
	pop	ax
	pop 	bp
        ret
_EscriureString ENDP



;;  *******************************************************************
;;    void EsborrarPantalla (BYTE atribut);
;;
;;          Subrutina d'esborrat de la pantalla:
;;              omple tota la pantalla de carcters blancs (' ', espai,
;;              codi ASCII 32) amb l'atribut que se li indica. La infor-
;;              maci d'on es troba mapejada la pantalla, l'obt de la
;;              variable SegmentPant. La seva implementaci pot utilit-
;;              zar la instrucci de tractament de "strings" REP STOSW.
;;  *******************************************************************

_EsborrarPantalla PROC FAR
        push    bp
        mov     bp, sp
        push	ax		;Guardem el registres que modificarem
        push	es
        push	di
        push	cx
        
        mov	ah, [bp+6]	;Atribut
        mov	al, ' '		;Omplirem la pant. d'espais en blanc
	mov     es, SegmentPant
	mov	di, 0
	mov	cx, 80*25	;Repetirem stosw tantes vegades com posicions
	cld			;Direccio positiva
	rep stosw		;hi ha a la pantalla
	
	pop	cx		;Restaurem el registres modificats
	pop	di
	pop	es
	pop	ax
	pop	bp
        ret
_EsborrarPantalla ENDP



;;  *******************************************************************
;;    WORD QuinCarAtr (BYTE fila, BYTE columna);
;;
;;          Subrutina de lectura d'un carcter de la pantalla:
;;              accedeix a la posici de pantalla indicada i retorna la
;;              parella (carcter, atribut) que es troba en aquella po-
;;              sici, el carcter a la part baixa (LSB) del resultat i
;;              l'atribut a la part alta (MSB). La informaci d'on es
;;              troba mapejada la pantalla, l'obt de SegmentPant.
;;  *******************************************************************

_QuinCarAtr     PROC FAR
        push    bp
        mov     bp, sp
        push    bx
        push    es

            ; Clcul del desplaament respecte a l'inici de pantalla
        mov     al, [bp+6]      ; AL <- fila
        mov     bl, 80
        mul     bl              ; AX <- fila*80
        mov     bl, [bp+8]      ; BL <- columna
        mov     bh, 0           ; BX <- columna
        add     bx, ax          ; BX <- fila*80+columna
        shl     bx, 1           ; BX <- (fila*80+columna)*2
        mov     es, SegmentPant
		; En es:[bx] tenim la posicio de pantalla
	mov	ax, es:[bx]	; La rutina retorna l'atribut i el caracter
				; de la posicio especificada mitjanant AX
	pop	es
	pop	bx
	pop	bp
        ret
_QuinCarAtr     ENDP



PANTALLA_CODI ENDS

        END
