;;  -------------------------------------------------------------------
;;  NOM:        TECL_INT.ASM
;;  CREAT PER:  Sergio Blanco Cuaresma
;;		Francesc Galiano Baixauli
;;  DATA:       31/05/2000
;;  DESCRIPCIO: Subrutines (inicialitzacio, lectura i finalitzacio) de
;;              teclat per interrupci amb la seva RSI corresponent
;;		**Corregida**
;;  -------------------------------------------------------------------

INCLUDE vect_int.inc    ; Subrutina _CanviarVectInt

;       A continuaci s'acostuma a posar les DECLARACIONS "PUBLIQUES":
;       quines dades i funcions s'han de "veure" externament

PUBLIC _IniTeclatINT, _FiTeclatINT, _HiHaTecla, _LlegirTeclatINT

;       Desprs s'acostuma a posar les DEFINICIONS DELS SEGMENTS de dades,
;       grups i segment de codi. El nom del segment de codi pot ser qual-
;       sevol (no s el cas dels segments de dades i el grup DGROUP)

IRQ_TECLAT EQU 1
INT_TECLAT EQU IRQ_TECLAT+8

        ; CONTROLADOR DE TECLAT (a partir del 286):
KTECL_DADES     EQU     60h     ; Registre de DADES del teclat


;       Segments de dades: _DATA (inicialitzades) i _BSS (no inicialitzades)

_DATA   SEGMENT WORD PUBLIC 'DATA'
_DATA   ENDS

_BSS    SEGMENT WORD PUBLIC 'BSS'
	SCtecla		DB	?	;Tecla pulsada
	AntigaRSI   	DD 	?
_BSS    ENDS

        ; els dos segments anteriors s'uneixen en un sol grup: DGROUP
DGROUP  GROUP   _DATA, _BSS

        ; declaraci d'on es suposa que apunten els registres de segment
ASSUME cs:TeclatINTCodi, ds:DGROUP


        ; segment de codi: pot tenir qualsevol nom (el segment, no el que
        ; posa 'CODE') per ha d'estar definit de la segent forma:

TeclatINTCodi SEGMENT BYTE PUBLIC 'CODE'


;; -------------------------------------------------------------------------
;; void IniTeclatINT (void);
;;
;; Realitza les inicialitzacions de variables (SCtecla) i dispusitius
;; (carregar la RSI del teclat a la posici corresponent del vector d'int.)
;; -------------------------------------------------------------------------
_IniTeclatINT PROC FAR
        push    ax              ; guardar registres que es modifiquen
        push	dx
        mov	SCtecla, 0
        ;Canviem vector d'int.
        mov     ax, SEG RSIteclat
        push    ax
        mov     ax, OFFSET RSIteclat
        push    ax
        mov     al, INT_TECLAT
        push    ax
        call    _CanviarVectInt         ; AntigaRSI <-- CanvVI (8, RSIrell);
        add     sp, 6  		;Ens oblidem del parametres guardats a la pila
        ;L'adrea retornada es la original que guardarem
        mov     WORD PTR [AntigaRSI], ax	;Desplaament
        mov     WORD PTR [AntigaRSI+2], dx	;Segment
        pop	dx
        pop	ax
        ret
_IniTeclatINT ENDP

;; -------------------------------------------------------------------------
;; void FiTeclatINT (void);
;;
;; Desfa les inicialitzacions de dispositius (restaura la RSI del teclat
;; original)
;; -------------------------------------------------------------------------
_FiTeclatINT PROC FAR
	push	ax
	push	dx
	;Recuperem l'adrea de l'RSI original
	mov     ax, WORD PTR [AntigaRSI+2]	;Segment
	mov     dx, WORD PTR [AntigaRSI]	;Desplaament
        push	ax
	push	dx
        mov     al, INT_TECLAT
        push    ax
        call    _CanviarVectInt         ; AntigaRSI <-- CanvVI (8, RSIrell);
        add     sp, 6  		;Ens oblidem del parametres guardats a la pila
        pop	dx
        pop	ax
        ret
_FiTeclatINT ENDP


;; -------------------------------------------------------------------------
;; BYTE HiHaTecla (void);
;;
;; Esbrina si s'ha pitjat alguna tecla (consultant la variable global
;; SCtecla)
;; -------------------------------------------------------------------------
_HiHaTecla PROC FAR
	;Si no hi ha tecla retornem 0
	cmp	SCtecla, 0
	jne	activo
	mov	al, 0
	jmp	fsi
activo:			;Si hi ha tecla retornem 1
	mov	al, 1
fsi:	
	ret
_HiHaTecla ENDP


;; -------------------------------------------------------------------------
;; BYTE LlegirTeclatINT (void);
;;
;; Retorna el scancode de la tecla que s'hagi pitjat
;; -------------------------------------------------------------------------
_LlegirTeclatINT PROC FAR

	;Mentre no hi hagui una tecla disponible no fer res
mentre:	cmp	BYTE PTR SCtecla, 0
	je	mentre
	
	;Un cop tenim la tecla obtenim el seu ScanCode
	pushf
	cli
	mov	al, SCtecla
	mov	SCtecla, 0
	popf
	ret        
_LlegirTeclatINT ENDP



;; -------------------------------------------------------------------------
;; static void RSIteclat (void);
;;
;; RSI del teclat (guarda els makes del teclat, ignora els breaks)
;; -------------------------------------------------------------------------
RSIteclat PROC FAR
	push	ax
	push	ds
	
	mov     ax, DGROUP      ; Inicialitzar DS per poder accedir
        mov     ds, ax          ;       al segment de dades (DGROUP)
        
	; S'activat la RSI, per tant hi ha una tecla per llegir
	in      al, KTECL_DADES         ; AL <- "Scancode" Tecla polsada
        test    al, 10000000b           ; Make/Break?
        jnz     EsBreak                 ; Ignorar Breaks
        mov	SCtecla, al
EsBreak:
                ; EOI
        mov     al, 20h
        out     20h, al
        pop	ds
	pop	ax
	iret
RSIteclat ENDP

TeclatINTCodi ENDS

        END     ; final de fitxer
